@@ -4,6 +4,7 @@ import gazu
 import getpass
 import pathlib
 import fnmatch
+import re
 
 from kabaret import flow
 from kabaret.app.ui.gui.widgets.flow.flow_view import (
@@ -36,7 +37,7 @@ from .site import (
     Synchronize, GotoCurrentSiteQueue,
     WorkingSites, ExchangeSites,
     MinioFileUploader, MinioFileDownloader,
-    RequestRevisions,
+    RequestRevisions, MultiRequestRevisions,
 )
 from .file import TrackedFile, Revision
 from .kitsu import KitsuAPIWrapper, KitsuBindings
@@ -77,9 +78,11 @@ class LoginPageWidget(CustomPageWidget):
 
         self.setLayout(grid_layout)
 
-        self.button_login.clicked.connect(self.on_button_login_clicked)
+        self.button_login.clicked.connect(self.on_connection_validated)
+        self.lineedit_kitsu_id.returnPressed.connect(self.on_connection_validated)
+        self.lineedit_kitsu_password.returnPressed.connect(self.on_connection_validated)
 
-    def on_button_login_clicked(self):
+    def on_connection_validated(self):
         kitsu_id = self.lineedit_kitsu_id.text()
         kitsu_password = self.lineedit_kitsu_password.text()
         logged_in = self.session.cmds.Flow.call(
@@ -212,6 +215,60 @@ class WorkingSiteChoiceValue(flow.values.ChoiceValue):
         return self.root().project().get_working_sites().mapped_names()
 
 
+class CleanZipFiles(flow.Action):
+
+    ICON = ('icons.libreflow', 'clean')
+
+    def __init__(self, parent, name):
+        super(CleanZipFiles, self).__init__(parent, name)
+        self._paths = []
+
+    def get_buttons(self):
+        self.message.set('<h2>Remove ZIP files</h2>')
+        return ['Confirm', 'Search', 'Cancel']
+    
+    def run(self, button):
+        if button == 'Cancel':
+            return
+        elif button == 'Search':
+            root_project = self.root().project().get_root().replace('\\', '/')
+            self._paths = []
+            file_count = 0
+            file_size = 0
+            
+            root_lib = os.path.join(root_project, 'lib').replace('\\', '/')
+            root_film = os.path.join(root_project, 'siren').replace('\\', '/')
+            
+            for root_path in (root_lib, root_film):
+                for root, _, files in os.walk(root_path):
+                    norm_root = root.replace('\\', '/').replace(root_project, '')
+                    
+                    for f in files:
+                        match = re.search('lib/(chars|props|sets)/.*/.*/.*/.*/v\d\d\d', norm_root)
+                        match = match or re.search('/siren/sq\d\d/sc\d\d\d\d/.*/.*/v\d\d\d', norm_root)
+                        
+                        if match is not None and os.path.splitext(f)[1] == '.zip':
+                            path = os.path.join(root, f)
+                            file_count += 1
+                            file_size += os.path.getsize(path)
+                            self._paths.append(path)
+            
+            self.message.set((
+                '<h2>Remove ZIP files</h2>'
+                '<b>%i</b> ZIP files found in this project (~%03f Gb).'
+                '<h3><font color="#D5000D">Remove all ?</font></h3>' % (file_count, file_size * 0.000000001)
+            ))
+
+            return self.get_result(close=False)
+        else:
+            for p in self._paths:
+                try:
+                    self.root().session().log_warning('Remove file %s' % p)
+                    os.remove(p)
+                except Exception:
+                    self.root().session().log_warning('Failed to remove file %s' % p)
+
+
 class Synchronization(flow.Object):
     ICON = ("icons.libreflow", "sync_arrow")
     
@@ -219,6 +276,7 @@ class Synchronization(flow.Object):
     synchronize_files = flow.Child(SynchronizeFiles)
     show_jobs = flow.Child(GotoCurrentSiteQueue)
     request_revisions = flow.Child(RequestRevisions)
+    multi_request_revisions = flow.Child(MultiRequestRevisions)
 
     def summary(self):
         current_site = self._project.get_current_site()
@@ -300,6 +358,7 @@ class ProjectSettings(flow.Object):
 
     default_files = flow.Child(DefaultFilesMap)
     non_editable_files = flow.Param("")
+    clean_zip_files = flow.Child(CleanZipFiles).ui(label='Clean ZIP files')
 
 
 class ExchangeSiteChoiceValue(flow.values.ChoiceValue):
