@@ -10,7 +10,7 @@ from .departments import Department
 from .maputils import ItemMap, CreateItemAction, ClearMapAction
 from .lib import AssetDependency, DropAssetAction  # , KitsuSettingsView
 from .kitsu import KitsuSequence, KitsuShot, UpdateItemsKitsuSettings
-from .site import RequestRevisions, SiteJobsPoolNames, ActiveSiteChoiceValue, ActiveSitesMultichoiceValue, RequestedRevisions
+from .site import RequestRevisions, MultiRequestRevisions, MultiRequestedRevisions, SiteJobsPoolNames, ActiveSiteChoiceValue, ActiveSitesMultichoiceValue, RequestedRevisions
 from .dependency import get_dependencies
 
 
@@ -74,31 +74,23 @@ class ShotElements(flow.values.MultiChoiceValue):
         return choices
 
 
-class RequestShot(RequestRevisions):
+class DependencyOptions(flow.Object):
     
-    _shot = flow.Parent()
-    _sequence = flow.Parent(3)
+    request_dependencies = flow.SessionParam(True).ui(editor='bool')
+    include_last_revisions = flow.SessionParam(False).ui(
+        editor='bool',
+        tooltip='Include the latest revision of each dependency')
+
+
+class ShotElementSelection(flow.Object):
+    
+    _shot = flow.Parent(2)
+    _sequence = flow.Parent(4)
     select_all_elements = flow.SessionParam(False).ui(editor='bool').watched()
-    elements = flow.Param([], ShotElements).watched().ui(label='Elements to request')
-    last_only = flow.SessionParam(True).watched().ui(hidden=True)
-    predictive_only = flow.SessionParam(False).ui(
-        editor='bool',
-        tooltip='Include predictive dependencies only'
-    )
-    latest = flow.SessionParam(True).ui(
-        editor='bool',
-        label='Latest published work',
-        editable=False,
-        hidden=True,
-    )
-    revisions = flow.Child(RequestedRevisions).ui(expanded=True)
-    pattern = flow.SessionParam('').watched().ui(
-        placeholder='Revision oid pattern',
-        hidden=True,
-    )
+    elements = flow.Param([], ShotElements).watched()
     
     def __init__(self, parent, name):
-        super(RequestShot, self).__init__(parent, name)
+        super(ShotElementSelection, self).__init__(parent, name)
         self._kitsu_casting = None
     
     def _get_casting(self):
@@ -110,18 +102,12 @@ class RequestShot(RequestRevisions):
 
         return casting
     
-    def _ensure_kitsu_casting(self):
+    def ensure_kitsu_casting(self):
         if self._kitsu_casting is None:
             self._kitsu_casting = self._get_casting()
         
         return self._kitsu_casting
     
-    def get_buttons(self):
-        self.message.set('<h2>Request %s%s</h2>' % (self._sequence.name(), self._shot.name()))
-        self._ensure_kitsu_casting()
-        
-        return super(RequestShot, self).get_buttons()
-    
     def child_value_changed(self, child_value):
         if child_value is self.select_all_elements:
             if self.select_all_elements.get():
@@ -129,94 +115,126 @@ class RequestShot(RequestRevisions):
             else:
                 self.elements.set([])
     
-    def compute_oids(self):
+    def compute_oids(self, include_dependencies=False, latest_dependencies=False):
         elements = self.elements.get()
         template = self.root().project().admin.dependency_templates['shot']
         bindings = self.root().project().kitsu_bindings()
         deps = template.get_dependencies()
         
-        kitsu_casting = self._ensure_kitsu_casting()
+        kitsu_casting = self.ensure_kitsu_casting()
         oids = set()
         
         for element in self.elements.get():
             dep_data = deps[element]
             kitsu_data = dep_data.get('kitsu', None)
             
-            if kitsu_data is not None:
-                # Kitsu dependencies
-                if kitsu_data['entity'] == 'Asset':
-                    for asset_name, asset_data in kitsu_casting.items():
-                        asset_type = asset_data['type']
-                        
-                        if asset_type != kitsu_data['type']:
-                            continue
-                        
-                        files_data = template.get_dependency_files(element)
-                        asset_oid = bindings.get_asset_oid(asset_name)
-                        
-                        # Get files
-                        for file_name, file_data in files_data.items():
-                            rev_oids = self._get_revision_oids(asset_oid, file_name, file_data)
-                            # pprint(oids)
-                            oids = oids.union(set(rev_oids))
+            if kitsu_data is not None and kitsu_data.get('entity', None) == 'Asset':
+                # Assets
+                for asset_name, asset_data in kitsu_casting.items():
+                    asset_type = asset_data['type']
+                    
+                    if asset_type != kitsu_data['type']:
+                        continue
+                    
+                    files_data = template.get_dependency_files(element)
+                    asset_oid = bindings.get_asset_oid(asset_name)
+                    
+                    # Get files
+                    for file_name, file_data in files_data.items():
+                        rev_oids = self._get_revision_oids(asset_oid, file_name, file_data, include_dependencies, latest_dependencies)
+                        # pprint(oids)
+                        oids = oids.union(set(rev_oids))
             else:
-                # Default dependencies
+                # Shots
                 files_data = template.get_dependency_files(element)
                 
                 for file_name, file_data in files_data.items():
-                    rev_oids = self._get_revision_oids(self._shot.oid(), file_name, file_data)
+                    rev_oids = self._get_revision_oids(self._shot.oid(), file_name, file_data, include_dependencies, latest_dependencies)
                     oids = oids.union(set(rev_oids))
         
         return sorted(list(oids))
     
-    def _get_revision_oids(self, root_oid, file_name, file_data):
+    def _get_revision_oids(self, root_oid, file_name, file_data, include_dependencies=False, latest_dependencies=False):
+        # Check if given file exists
         file_oid = "%s/departments/%s/files/%s" % (
             root_oid,
             file_data['department'],
             file_name.replace('.', '_')
         )
-        rev_name_pattern = file_data.get('revision', '[last]')
-        
-        rev_oids = ["%s/history/revisions/%s" % (
-            file_oid,
-            rev_name_pattern,
-        )]
-        
-        needs_deps = file_data.get('needs_deps', False)
-        
-        # Get file dependencies if needed
-        if not needs_deps:
-            return rev_oids
-        
-        if rev_name_pattern == '[last]':
-            rev_name = None
-        else:
-            rev_name = rev_name_pattern
         
         if not self.root().session().cmds.Flow.exists(file_oid):
-            print("ERROR: File %s not defined in the flow" % file_oid)
-            return rev_oids
+            self.root().session().log_warning('Unknown file with oid ' + file_oid)
+            return []
+        
+        revision_name = file_data.get('revision', '[last]')
+        revision_oids = [file_oid + '/history/revisions/' + revision_name]
+        
+        # Return if dependencies are not requested
+        if not include_dependencies:
+            return revision_oids
+        
+        if revision_name == '[last]':
+            revision_name = None
+        
+        # Get dependencies
         
         file_object = self.root().get_object(file_oid)
-        file_deps = get_dependencies(
-            file_object,
+        dependencies = get_dependencies(
+            leaf=file_object,
             predictive=True,
-            real=not self.predictive_only.get(),
-            revision_name=rev_name
-        )
+            real=True,
+            revision_name=revision_name)
         
-        for dep in file_deps:
-            if dep['in_breakdown']:
-                rev_oid = dep['revision_oid']
-                if rev_oid is not None:
-                    rev_oids.append(rev_oid)
-                else:
-                    print((
-                        "WARNING: dependency %s in breakdown "
-                        "but not defined in the flow" % dep['entity_oid']
-                    ))
+        for d in dependencies:
+            if not d['in_breakdown']:
+                self.root().session().log_warning(
+                    'Dependency %s not in breakdown ' % d['entity_oid']
+                )
+                continue
+            
+            revision_oid = d['revision_oid']
+            
+            if revision_oid is None:
+                self.root().session().log_warning((
+                    'Dependency %s in breakdown '
+                    'but not defined in the flow' % d['entity_oid']
+                ))
+                continue
+            
+            revision_oids.append(revision_oid)
+            
+        # Get predictive dependencies if required
+        if latest_dependencies:
+            real_deps = get_dependencies(
+                leaf=file_object,
+                predictive=True,
+                revision_name=revision_name)
+            
+            for d in real_deps:
+                revision_oid = d['revision_oid']
+                
+                if revision_oid is not None and not revision_oid in revision_oids:
+                    revision_oids.append(revision_oid)
         
-        return rev_oids
+        return revision_oids
+
+
+class RequestShot(RequestRevisions):
+    
+    _shot = flow.Parent()
+    _sequence = flow.Parent(3)
+    selection = flow.Child(ShotElementSelection).ui(expanded=True)
+    dependency_options = flow.Child(DependencyOptions).ui(expanded=True)
+    revisions = flow.Child(RequestedRevisions).ui(expanded=True)
+    pattern = flow.SessionParam('').ui(
+        placeholder='Revision oid pattern',
+        hidden=True)
+    
+    def get_buttons(self):
+        self.message.set('<h2>Request %s%s</h2>' % (self._sequence.name(), self._shot.name()))
+        self.selection.ensure_kitsu_casting()
+        
+        return super(RequestShot, self).get_buttons()
     
     def run(self, button):
         if button == 'Close':
@@ -224,23 +242,59 @@ class RequestShot(RequestRevisions):
         elif button == 'Request':
             return super(RequestShot, self).run(button)
         
-        oids = self.compute_oids()
+        oids = self.selection.compute_oids(
+            include_dependencies=self.dependency_options.request_dependencies.get(),
+            latest_dependencies=self.dependency_options.include_last_revisions.get()
+        )
         self.pattern.set(';'.join(oids))
         
         return super(RequestShot, self).run(button)
 
 
+class MultiRequestShot(MultiRequestRevisions):
+
+    _shot = flow.Parent()
+    _sequence = flow.Parent(3)
+    selection = flow.Child(ShotElementSelection).ui(expanded=True)
+    dependency_options = flow.Child(DependencyOptions).ui(expanded=True)
+    revisions = flow.Child(MultiRequestedRevisions).ui(expanded=True)
+    pattern = flow.SessionParam('').ui(
+        placeholder='Revision oid pattern',
+        hidden=True)
+    
+    def get_buttons(self):
+        self.message.set('<h2>Request %s%s towards multiple sites</h2>' % (self._sequence.name(), self._shot.name()))
+
+        return super(MultiRequestShot, self).get_buttons()
+    
+    def run(self, button):
+        if button == 'Close':
+            return
+        elif button == 'Request':
+            return super(MultiRequestShot, self).run(button)
+        
+        oids = self.selection.compute_oids(
+            include_dependencies=self.dependency_options.request_dependencies.get(),
+            latest_dependencies=self.dependency_options.include_last_revisions.get()
+        )
+        self.pattern.set(';'.join(oids))
+        
+        return super(MultiRequestShot, self).run(button)
+
+
 class Shot(KitsuShot):
 
     ICON = ("icons.flow", "shot")
 
     _sequence = flow.Parent(2)
-    request = flow.Child(RequestShot)
     settings = flow.Child(ContextualView).ui(hidden=True)
     # casting = flow.Child(Casting)
 
     description = flow.Param("")
     departments = flow.Child(ShotDepartments).ui(expanded=True)
+    
+    request = flow.Child(RequestShot)
+    multi_request = flow.Child(MultiRequestShot)
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
@@ -301,67 +355,36 @@ class SequenceElements(flow.values.MultiChoiceValue):
 
 class ShotsMultichoiceValue(flow.values.MultiChoiceValue):
     
-    _sequence = flow.Parent(2)
+    _sequence = flow.Parent(3)
     
     def choices(self):
         return self._sequence.shots.mapped_names()
 
 
-class RequestSequence(RequestRevisions):
-
-    _sequence = flow.Parent()
+class SequenceElementSelection(flow.Object):
+    
+    _sequence = flow.Parent(2)
     select_all_elements = flow.SessionParam(False).ui(editor='bool').watched()
     elements = flow.Param([], ShotElements).ui(label="Elements to request")
     select_all_shots = flow.SessionParam(False).ui(editor='bool').watched()
     shots = flow.Param([], ShotsMultichoiceValue)
-    last_only = flow.SessionParam(True).ui(editor='bool')
-    predictive_only = flow.SessionParam(False).ui(
-        editor='bool',
-        tooltip='Include predictive dependencies only'
-    )
-    latest = flow.SessionParam(True).ui(
-        editor='bool',
-        label='Latest published work',
-        editable=False,
-        hidden=True,
-    )
-    revisions = flow.Child(RequestedRevisions).ui(expanded=True)
-    pattern = flow.SessionParam('').watched().ui(
-        placeholder='Revision oid pattern',
-        hidden=True,
-    )
     
-    def _revert_to_defaults(self):
+    def revert_to_defaults(self):
         self.elements.revert_to_default()
-        self.last_only.revert_to_default()
         self.shots.revert_to_default()
     
-    def compute_oids(self):
+    def compute_oids(self, include_dependencies=False, latest_dependencies=False):
         oids = []
-        
         elements = self.elements.get()
-        last_only = self.last_only.get()
-        predictive_only = self.predictive_only.get()
-        latest = self.latest.get()
         
         for shot_name in self.shots.get():
             shot = self._sequence.shots[shot_name]
+            shot.request.selection.elements.set(elements)
             
-            shot.request.elements.set(elements)
-            shot.request.last_only.set(last_only)
-            shot.request.predictive_only.set(predictive_only)
-            shot.request.latest.set(latest)
-            
-            oids += shot.request.compute_oids()
+            oids += shot.request.selection.compute_oids(include_dependencies, latest_dependencies)
         
         return oids
     
-    def get_buttons(self):
-        self.message.set('<h2>Request %s</h2>' % self._sequence.name())
-        self._revert_to_defaults()
-
-        return super(RequestSequence, self).get_buttons()
-    
     def child_value_changed(self, child_value):
         if child_value is self.select_all_shots:
             if self.select_all_shots.get():
@@ -373,6 +396,24 @@ class RequestSequence(RequestRevisions):
                 self.elements.set(self.elements.choices())
             else:
                 self.elements.set([])
+
+
+class RequestSequence(RequestRevisions):
+
+    _sequence = flow.Parent()
+    selection = flow.Child(SequenceElementSelection).ui(expanded=True)
+    dependency_options = flow.Child(DependencyOptions).ui(expanded=True)
+    revisions = flow.Child(RequestedRevisions).ui(expanded=True)
+    pattern = flow.SessionParam('').ui(
+        placeholder='Revision oid pattern',
+        hidden=True,
+    )
+    
+    def get_buttons(self):
+        self.message.set('<h2>Request %s</h2>' % self._sequence.name())
+        self.selection.revert_to_defaults()
+
+        return super(RequestSequence, self).get_buttons()
     
     def run(self, button):
         if button == 'Close':
@@ -380,12 +421,46 @@ class RequestSequence(RequestRevisions):
         elif button == 'Request':
             return super(RequestSequence, self).run(button)
         
-        oids = self.compute_oids()
+        oids = self.selection.compute_oids(
+            include_dependencies=self.dependency_options.request_dependencies.get(),
+            latest_dependencies=self.dependency_options.include_last_revisions.get()
+        )
         self.pattern.set(';'.join(oids))
         
         return super(RequestSequence, self).run(button)
 
 
+class MultiRequestSequence(MultiRequestRevisions):
+
+    _sequence = flow.Parent()
+    selection = flow.Child(SequenceElementSelection).ui(expanded=True)
+    dependency_options = flow.Child(DependencyOptions).ui(expanded=True)
+    revisions = flow.Child(MultiRequestedRevisions).ui(expanded=True)
+    pattern = flow.SessionParam('').ui(
+        placeholder='Revision oid pattern',
+        hidden=True)
+    
+    def get_buttons(self):
+        self.message.set('<h2>Request %s towards multiple sites</h2>' % self._sequence.name())
+        self.selection.revert_to_defaults()
+
+        return super(MultiRequestSequence, self).get_buttons()
+    
+    def run(self, button):
+        if button == 'Close':
+            return
+        elif button == 'Request':
+            return super(MultiRequestSequence, self).run(button)
+        
+        oids = self.selection.compute_oids(
+            include_dependencies=self.dependency_options.request_dependencies.get(),
+            latest_dependencies=self.dependency_options.include_last_revisions.get()
+        )
+        self.pattern.set(';'.join(oids))
+        
+        return super(MultiRequestSequence, self).run(button)
+
+
 class PlayblastItem(flow.Object):
 
     to_submit = flow.SessionParam(False).ui(editor='bool').watched()
@@ -815,6 +890,109 @@ class RenderSequencePlayblasts(ProcessSequenceFiles):
         ui['custom_page'] = 'libreflow.utils.ui.RenderSequencePlayblastsWidget'
 
 
+class UploadSequencePlayblastsToKitsu(ProcessSequenceFiles):
+    
+    ICON = ('icons.libreflow', 'kitsu-upload')
+    
+    def get_task_statutes(self):
+        return self.root().project().kitsu_api().get_task_statutes()
+    
+    def get_kitsu_task_type(self, task_name):
+        dependencies = self.root().project().admin.dependency_templates['shot'].get_dependencies()
+        kitsu_data = dependencies[task_name].get('kitsu', None)
+        
+        if kitsu_data is None:
+            return None
+        
+        return kitsu_data.get('task_type', 'UNDEFINED')
+    
+    def upload_playblast_to_kitsu(self, file_oid, revision_name, task_type, target_task_status, comment):
+        file = self.root().get_object(file_oid)
+        file.upload_playblast.revision_name.set(revision_name)
+        file.upload_playblast.target_task_type.set(task_type)
+        file.upload_playblast.target_task_status.set(target_task_status)
+        file.upload_playblast.comment.set(comment)
+        
+        file.upload_playblast.run(None)
+    
+    def _compute_files_data(self):
+        # Search for playblast files
+        playblasts_files_by_task = self.get_shot_task_files(['*.mov'])
+        # print(blender_files_by_task)
+        
+        shots = self._sequence.shots
+        files_data = []
+        
+        for shot in shots.mapped_items():
+            tasks_files = []
+            
+            for task_name, task_files in playblasts_files_by_task:
+                files = []
+                default_kitsu_task_type = self.get_kitsu_task_type(task_name)
+                
+                for file_name, file_data in task_files:
+                    file_mapped_name = file_name.replace('.', '_')
+                    dept_name = file_data['department']
+                    file_oid = f'{shot.oid()}/departments/{dept_name}/files/{file_mapped_name}'
+                    
+                    if not self.root().session().cmds.Flow.exists(file_oid):
+                        # Skip if file does not exist in the flow
+                        continue
+                    
+                    file = self.root().get_object(file_oid)
+                    revision_statuses = file.get_revision_statuses(published_only=True)
+                    head = file.get_head_revision(sync_status='Available')
+                    
+                    default_revision = head.name() if head is not None else None
+                    
+                    if not revision_statuses:
+                        self.root().session().log_warning(
+                            f'File {file.oid()} has no published revision available'
+                        )
+                        continue
+                    
+                    task_types = file.upload_playblast.target_task_type.choices()
+                    
+                    if default_kitsu_task_type is not None:
+                        file.upload_playblast.target_task_type.set(default_kitsu_task_type)
+                        
+                        if len(task_types) == 1:
+                            # Site has no Kitsu priviledge, and single task type depends on file name
+                            # => Change to Kitsu task type specified by dependency
+                            task_types = [default_kitsu_task_type]
+                    else:
+                        file.upload_playblast.target_task_type.revert_to_default()
+                    
+                    files.append(dict(
+                        oid=file.oid(),
+                        name=file_name,
+                        shot=shot.name(),
+                        department=dept_name,
+                        revisions=revision_statuses,
+                        default_revision=default_revision,
+                        task_types=task_types,
+                        default_task_type=file.upload_playblast.target_task_type.get(),
+                        current_task_status=file.upload_playblast.current_task_status.get(),
+                        default_task_status=file.upload_playblast.target_task_status.get()
+                    ))
+                
+                tasks_files.append(dict(
+                    name=task_name,
+                    files=files
+                ))
+            
+            files_data.append(dict(
+                oid=shot.oid(),
+                name=shot.name(),
+                tasks=tasks_files
+            ))
+        
+        return files_data
+    
+    def _fill_ui(self, ui):
+        ui['custom_page'] = 'libreflow.utils.ui.UploadSequencePlayblastsWidget'
+
+
 class Sequence(KitsuSequence):
 
     ICON = ("icons.flow", "sequence")
@@ -826,7 +1004,9 @@ class Sequence(KitsuSequence):
     shots = flow.Child(Shots).ui(expanded=True)
     
     request = flow.Child(RequestSequence).ui(hidden=True)
+    multi_request = flow.Child(MultiRequestSequence)
     render_playblasts = flow.Child(RenderSequencePlayblasts)
+    upload_playblasts = flow.Child(UploadSequencePlayblastsToKitsu)
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
