@@ -148,6 +148,10 @@ class KitsuAPIWrapper(flow.Object):
         
         return task
     
+    def get_task_statutes(self):
+        task_statuses = gazu.task.all_task_statuses()
+        return [ts['name'] for ts in task_statuses]
+    
     def get_task_current_status(self, entity, task_type_name):
         task = self.get_task(entity, task_type_name)
         
@@ -268,6 +272,10 @@ class KitsuBindings(flow.Object):
     def get_shot_casting(self, shot_name, sequence_name):
         kitsu_api = self._kitsu_api()
         kitsu_casting = kitsu_api.get_shot_casting(shot_name, sequence_name)
+        
+        if kitsu_casting is None:
+            return {}
+        
         casting = dict()
 
         for asset in kitsu_casting:
@@ -343,8 +351,7 @@ class KitsuTaskStatus(flow.values.ChoiceValue):
     STRICT_CHOICES = False
     
     def choices(self):
-        task_statuses = gazu.task.all_task_statuses()
-        return [ts['name'] for ts in task_statuses]
+        return self.root().project().kitsu_api().get_task_statutes()
 
 
 class UpdateKitsuSettings(flow.Action):
