@@ -7,12 +7,13 @@ from datetime import datetime
 
 from kabaret import flow
 from kabaret.subprocess_manager.runner_factory import Runner, logger
-from kabaret.subprocess_manager.flow import RunAction
+# from kabaret.subprocess_manager.flow import RunAction
 
 from ..utils.kabaret.subprocess_manager.runner_factory import Runner
+from ..utils.kabaret.subprocess_manager.flow import RunAction
 
 
-CHOICES = ["blend", "kra", "png", "jpg", "txt", "abc", "mov", "psd", "aep", "zip", "mp4", "fbx", "ai", "json", "jsx"]
+CHOICES = ["blend", "kra", "png", "jpg", "txt", "abc", "mov", "psd", "aep", "zip", "mp4", "fbx", "ai", "json", "jsx", "obj"]
 CHOICES_ICONS = {
             "blend": ("icons.libreflow", "blender"),
             "kra": ("icons.libreflow", "krita"),
@@ -29,6 +30,7 @@ CHOICES_ICONS = {
             "fbx": ("icons.libreflow", "fbx"),
             "json": ("icons.libreflow", "json"),
             "jsx": ("icons.libreflow", "jsx"),
+            "obj": ("icons.libreflow", "3d-object"),
         }
 
 
@@ -141,6 +143,24 @@ class Krita(EditFileRunner):
         return [".kra", ".png", ".jpg"]
 
 
+class AfterEffects(EditFileRunner):
+
+    ICON = ("icons.libreflow", "afterfx")
+
+    @classmethod
+    def supported_extensions(cls):
+        return [".aep", ".png", ".jpg"]
+
+
+class AfterEffectsRender(EditFileRunner):
+
+    ICON = ("icons.libreflow", "afterfx")
+
+    @classmethod
+    def supported_extensions(cls):
+        return [".aep"]
+
+
 class VSCodium(EditFileRunner):
 
     ICON = ("icons.libreflow", "vscodium")
@@ -183,6 +203,15 @@ class PythonRunner(Runner):
         return False
 
 
+class MarkSequenceRunner(PythonRunner):
+    
+    TAGS = ['Mark image sequence']
+
+    def argv(self):
+        script_path = "%s/../scripts/mark_sequence.py" % os.path.dirname(__file__)
+        return [script_path] + self.extra_argv
+
+
 class SessionWorker(PythonRunner):
 
     def argv(self):
@@ -212,6 +241,23 @@ class LaunchSessionWorker(RunAction):
             self.launcher_oid(),
             self.launcher_exec_func_name()
         ]
+    
+    def run(self, button):
+        '''
+        Sets the environment variable which contains the runner executable path
+        before launching the runner.
+        '''
+        name, tags = self.runner_name_and_tags()
+        
+        rid = self.root().session().cmds.SubprocessManager.run(
+            runner_name=name,
+            tags=tags,
+            version=self.get_version(button),
+            label=self.get_run_label(),
+            extra_argv=self.extra_argv(),
+            extra_env=self.extra_env(),
+        )
+        return self.get_result(runner_id=rid)
 
 
 class RunnerChoiceValue(flow.values.ChoiceValue):
