@@ -399,9 +399,37 @@ class ActiveSiteChoiceValue(flow.values.SessionValue):
         return self._choices.get()
 
 
+class ActiveSiteAutoSelectChoiceValue(ActiveSiteChoiceValue):
+    
+    def choices(self):
+        choices = super(ActiveSiteAutoSelectChoiceValue, self).choices()
+        choices = ['Auto select'] + choices.copy()
+        
+        return choices
+
+
 class ActiveSitesMultichoiceValue(ActiveSiteChoiceValue):
     
     DEFAULT_EDITOR = 'multichoice'
+    
+    exclude_choice = flow.SessionParam()
+    
+    def choices(self):
+        if self._choices.get() is None:
+            working_sites = self.root().project().get_working_sites()
+            names = working_sites.get_site_names(use_custom_order=True, active_only=True)
+            self._choices.set(names)
+        
+        choices = self._choices.get().copy()
+        exclude_choice = self.exclude_choice.get()
+        
+        if exclude_choice is not None:
+            try:
+                choices.remove(exclude_choice)
+            except ValueError:
+                pass
+        
+        return choices
 
 
 class ActiveSiteRevisionAvailableChoiceValue(flow.values.ChoiceValue):
@@ -440,13 +468,16 @@ class SiteSelection(flow.Object):
     
     source_site = flow.Param(None, ActiveSiteChoiceValue).watched()
     target_site = flow.Param(None, ActiveSiteChoiceValue).watched()
+    auto_select_enabled = flow.SessionParam(False).watched().ui(
+        editor='bool',
+        hidden=True)
     
     def child_value_changed(self, child_value):
-        if child_value in (self.source_site, self.target_site):
+        if child_value in (self.source_site, self.target_site, self.auto_select_enabled):
             self.touch()
     
     def summary(self):
-        if self.source_site.get() == self.target_site.get():
+        if self.source_site.get() == self.target_site.get() and not self.auto_select_enabled.get():
             return '⚠️ Source and target sites can\'t be the same !'
         else:
             return ''
@@ -533,7 +564,11 @@ class RequestAs(flow.Action):
             )
             self.root().project().synchronization.touch()
             return self.get_result()
-
+        
+        # If enabled, automatically select source site
+        if self.sites.auto_select_enabled.get():
+            source_site_name = self._revision.site.get()
+        
         source_site = sites[source_site_name]
 
         # Check if the source version upload is not already requested
@@ -1205,7 +1240,7 @@ class RequestRevisions(flow.Action):
     
     ICON = ('icons.gui', 'share-option')
 
-    pattern = flow.SessionParam("").watched().ui(
+    pattern = flow.SessionParam("").ui(
         placeholder="Revision oid pattern"
     )
     
@@ -1342,8 +1377,124 @@ class RequestRevisions(flow.Action):
             
             obj.request_as.sites.source_site.set(source_site)
             obj.request_as.sites.target_site.set(target_site)
+            obj.request_as.include_dependencies.set(False)
             obj.request_as.run(None)
+        
+        self.revisions.touch()
+
+        return self.get_result(close=False)
+
+
+class SiteMultiSelection(flow.Object):
+    
+    source_site = flow.Param(None, ActiveSiteAutoSelectChoiceValue).watched()
+    target_sites = flow.Param([], ActiveSitesMultichoiceValue)
+    
+    def child_value_changed(self, child_value):
+        if child_value is self.source_site:
+            source_site = self.source_site.get()
             
+            if source_site == 'Auto select':
+                self.target_sites.exclude_choice.set(None)
+            else:
+                self.target_sites.exclude_choice.set(self.source_site.get())
+            
+            self.target_sites.touch()
+
+
+class MultiRequestedRevisions(RequestedRevisions):
+    
+    STYLE_BY_STATUS = {
+        'Available': ('#45cc3d', ('icons.libreflow', 'checked-symbol-colored')),
+        'Requested': ('#b9c2c8', ('icons.libreflow', 'exclamation-sign-colored')),
+        'NotAvailable': ('#cc3b3c', ('icons.libreflow', 'blank')),
+        'Undefined': ('#cc3b3c', ('icons.libreflow', 'unavailable')),
+    }
+    
+    request_action = flow.Parent()
+    
+    def columns(self):
+        return ['Revision'] + self.request_action.sites.target_sites.get()
+    
+    def _fill_row_cells(self, row, item):
+        row['Revision'] = item.oid()
+        
+        for col in self.columns()[1:]:
+            row[col] = ''
+    
+    def _fill_row_style(self, style, item, row):
+        style['Revision_icon'] = ('icons.libreflow', 'blank')
+        
+        if not hasattr(item, 'request_as'):
+            style['Revision_icon'] = self.STYLE_BY_STATUS['Undefined'][1]
+            return
+        
+        for site_name in self.request_action.sites.target_sites.get():
+            status = item.get_sync_status(site_name=site_name)
+            style[site_name + '_icon'] = self.STYLE_BY_STATUS[status][1]
+
+
+class MultiRequestRevisions(RequestRevisions):
+    
+    ICON = ('icons.libreflow', 'multi-share-option')
+    
+    sites = flow.Child(SiteMultiSelection).ui(expanded=True)
+    revisions = flow.Child(MultiRequestedRevisions).ui(expanded=True)
+    
+    def get_buttons(self):
+        self.sites.source_site.set(self.root().project().get_current_site().name())
+        target_sites_choices = self.sites.target_sites.choices()
+        
+        if not target_sites_choices:
+            msg = self.message.get()
+            if msg is None:
+                msg = ''
+            
+            msg += (
+                '<h3><font color="#D5000D">Making requests is not possible since '
+                'there is no other site defined for this project.</font></h3>'
+            )
+            self.message.set(msg)
+            
+            return ['Close']
+
+        return ['Request', 'Refresh revision list', 'Close']
+    
+    def run(self, button):
+        if button == 'Close':
+            return
+        elif button == 'Refresh revision list':
+            self.revisions.oids.revert_to_default()
+            oids = self._get_oid(self.pattern.get())
+            self.revisions.oids.set(oids)
+            self.revisions.touch()
+            
+            return self.get_result(close=False)
+        
+        source_site = self.sites.source_site.get()
+        auto_select_enabled = (source_site == 'Auto select')
+        
+        for obj in self.revisions.mapped_items():
+            # Skip objects which are not file revisions
+            if not hasattr(obj, 'request_as'):
+                continue
+            # Skip revisions which are not available on selected source site
+            if not auto_select_enabled and obj.get_sync_status(site_name=source_site) != 'Available':
+                continue
+            
+            for target_site in self.sites.target_sites.get():
+                # Skip revisions which are already available on selected target site
+                status = obj.get_sync_status(site_name=target_site)
+                if status == 'Requested' or status == 'Available':
+                    continue
+                
+                if not auto_select_enabled:
+                    obj.request_as.sites.source_site.set(source_site)
+                
+                obj.request_as.sites.target_site.set(target_site)
+                obj.request_as.sites.auto_select_enabled.set(auto_select_enabled)
+                obj.request_as.include_dependencies.set(False)
+                obj.request_as.run(None)
         
         self.revisions.touch()
 
