@@ -333,7 +333,7 @@ class AddEnvVarAction(flow.Action):
         json.dump(env, f, indent=4, sort_keys=True)
         f.close()
 
-        os.environ[self.var_name.get()] = self.var_value.get()
+        #os.environ[self.var_name.get()] = self.var_value.get()
         self._env.touch()
 
 
@@ -342,7 +342,7 @@ class ChangeEnvVarValueAction(flow.Action):
     _var = flow.Parent()
     _env = flow.Parent(2)
 
-    var_value = flow.Param().ui(label="Value")
+    var_value = flow.Param("").ui(label="Value")
 
     def get_buttons(self):
         return ["Confirm", "Cancel"]
@@ -374,7 +374,8 @@ class EnvVar(flow.values.SessionValue):
     change_value = flow.Child(ChangeEnvVarValueAction)
 
     def update(self):
-        os.environ[self.name()] = self.get()
+        pass
+        #os.environ[self.name()] = self.get()
 
 
 class UserEnvironment(flow.DynamicMap):
